=== modified file 'branchview/linegraph.py'
--- branchview/linegraph.py	2008-01-13 14:12:49 +0000
+++ branchview/linegraph.py	2008-02-14 19:59:06 +0000
@@ -11,7 +11,8 @@
 
 from bzrlib.tsort import merge_sort
 
-def linegraph(repository, start, maxnum, broken_line_length=None):
+def linegraph(repository, start, maxnum, broken_line_length = None,
+              graph_data = True, mainline_only = False):
     """Produce a directed graph of a bzr repository.
 
     Returns a tuple of (line_graph, revid_index, columns_len) where
@@ -54,6 +55,10 @@
     else:
         merge_sorted_revisions = ()
     
+    if mainline_only:
+        merge_sorted_revisions = [elem for elem in merge_sorted_revisions \
+                                  if len(elem[3])==1 ]
+    
     revid_index = {}
     revno_index = {}
     
@@ -76,18 +81,6 @@
         if maxnum and rev_index >= maxnum:
             break
         revid_index[revid] = rev_index
-        revno_index[revno_sequence] = rev_index
-        
-        branch_id = revno_sequence[0:-1]
-        
-        branch_line = None
-        if branch_id not in branch_lines:
-            branch_line = []
-            branch_lines[branch_id] = branch_line
-        else:
-            branch_line = branch_lines[branch_id]
-        
-        branch_line.append(rev_index)
         
         parents = graph_parents[revid]
         for parent_revid in parents:
@@ -99,207 +92,223 @@
                           parents,
                           None,
                           revno_sequence])
+        
+        if graph_data:
+            revno_index[revno_sequence] = rev_index
+            
+            branch_id = revno_sequence[0:-1]
+            
+            branch_line = None
+            if branch_id not in branch_lines:
+                branch_line = []
+                branch_lines[branch_id] = branch_line
+            else:
+                branch_line = branch_lines[branch_id]
+            
+            branch_line.append(rev_index)        
 
-    branch_ids = branch_lines.keys()
-    
-    def branch_id_cmp(x, y):
-        """Compaire branch_id's first by the number of digits, then reversed
-        by their value"""
-        len_x = len(x)
-        len_y = len(y)
-        if len_x == len_y:
-            return -cmp(x, y)
-        return cmp(len_x, len_y)
-    
-    branch_ids.sort(branch_id_cmp)
-    # This will hold a tuple of (child_index, parent_index, col_index) for each
-    # line that needs to be drawn. If col_index is not none, then the line is
-    # drawn along that column, else the the line can be drawn directly between
-    # the child and parent because either the child and parent are in the same
-    # branch line, or the child and parent are 1 row apart.
-    lines = []
-    empty_column = [False for i in range(len(graph_parents))]
-    # This will hold a bit map for each cell. If the cell is true, then the
-    # cell allready contains a node or line. This use when deciding what column
-    # to place a branch line or line in, without it overlaping something else.
-    columns = [list(empty_column)]
-    
-    
-    for branch_id in branch_ids:
-        branch_line = branch_lines[branch_id]
-        
-        # Find the col_index for the direct parent branch. This will be the
-        # starting point when looking for a free column.
-        parent_col_index = 0
-        parent_index = None
-        if len(branch_id) > 1:
-            parent_revno = branch_id[0:-1]
-            if parent_revno in revno_index:
-                parent_index = revno_index[parent_revno]
-                parent_node = linegraph[parent_index][1]
-                if parent_node:
-                    parent_col_index = parent_node[0]
-                
-        
-        col_search_order = _branch_line_col_search_order(columns,
-                                                         parent_col_index)
-        color = reduce(lambda x, y: x+y, branch_id, 0)
-        cur_cont_line = []
-        
-        line_range = []
-        last_rev_index = None
-        for rev_index in branch_line:
-            if last_rev_index:
-                if broken_line_length and \
-                   rev_index - last_rev_index > broken_line_length:
-                    line_range.append(last_rev_index+1)
-                    line_range.append(rev_index-1)
-                else:
-                    line_range.extend(range(last_rev_index+1, rev_index))
-            
-            line_range.append(rev_index)
-            last_rev_index = rev_index
-        
-        if parent_index:
-            if broken_line_length and \
-               parent_index - last_rev_index > broken_line_length:
-                line_range.append(last_rev_index+1)
-            else:
-                line_range.extend(range(last_rev_index+1, parent_index))
-        
-        col_index = _find_free_column(columns,
-                                      empty_column,
-                                      col_search_order,
-                                      line_range)
-        node = (col_index, color)
-        for rev_index in branch_line:
-            linegraph[rev_index][1] = node
-            columns[col_index][rev_index] = True
-        
-        for rev_index in branch_line:
-            (sequence_number,
-                 revid,
-                 merge_depth,
-                 revno_sequence,
-                 end_of_merge) = merge_sorted_revisions[rev_index]
-            
-            linegraph[rev_index][4] = graph_children[revid]
-            col_index = linegraph[rev_index][1][0]
-            
-            for parent_revid in graph_parents[revid]:
-                if parent_revid in revid_index:
-                    
-                    parent_index = revid_index[parent_revid]                            
+    if graph_data:
+        branch_ids = branch_lines.keys()
+    
+        def branch_id_cmp(x, y):
+            """Compaire branch_id's first by the number of digits, then reversed
+            by their value"""
+            len_x = len(x)
+            len_y = len(y)
+            if len_x == len_y:
+                return -cmp(x, y)
+            return cmp(len_x, len_y)
+        
+        branch_ids.sort(branch_id_cmp)
+        # This will hold a tuple of (child_index, parent_index, col_index) for each
+        # line that needs to be drawn. If col_index is not none, then the line is
+        # drawn along that column, else the the line can be drawn directly between
+        # the child and parent because either the child and parent are in the same
+        # branch line, or the child and parent are 1 row apart.
+        lines = []
+        empty_column = [False for i in range(len(graph_parents))]
+        # This will hold a bit map for each cell. If the cell is true, then the
+        # cell allready contains a node or line. This use when deciding what column
+        # to place a branch line or line in, without it overlaping something else.
+        columns = [list(empty_column)]
+        
+        
+        for branch_id in branch_ids:
+            branch_line = branch_lines[branch_id]
+            
+            # Find the col_index for the direct parent branch. This will be the
+            # starting point when looking for a free column.
+            parent_col_index = 0
+            parent_index = None
+            if len(branch_id) > 1:
+                parent_revno = branch_id[0:-1]
+                if parent_revno in revno_index:
+                    parent_index = revno_index[parent_revno]
                     parent_node = linegraph[parent_index][1]
                     if parent_node:
                         parent_col_index = parent_node[0]
+                    
+            
+            col_search_order = _branch_line_col_search_order(columns,
+                                                             parent_col_index)
+            color = reduce(lambda x, y: x+y, branch_id, 0)
+            cur_cont_line = []
+            
+            line_range = []
+            last_rev_index = None
+            for rev_index in branch_line:
+                if last_rev_index:
+                    if broken_line_length and \
+                       rev_index - last_rev_index > broken_line_length:
+                        line_range.append(last_rev_index+1)
+                        line_range.append(rev_index-1)
                     else:
-                        parent_col_index = None
-                    col_search_order = \
-                            _line_col_search_order(columns,
-                                                   parent_col_index,
-                                                   col_index)
-                        
-                    # If this line is really long, break it.
-                    if len(branch_id) > 0 and \
-                       broken_line_length and \
-                       parent_index - rev_index > broken_line_length:
-                        child_line_col_index = \
-                            _find_free_column(columns,
-                                              empty_column,
-                                              col_search_order,
-                                              (rev_index + 1,))
-                        _mark_column_as_used(columns,
-                                             child_line_col_index,
-                                             (rev_index + 1,))
-                        
-                        # Recall _line_col_search_order to reset it back to
-                        # the beging.
+                        line_range.extend(range(last_rev_index+1, rev_index))
+                
+                line_range.append(rev_index)
+                last_rev_index = rev_index
+            
+            if parent_index:
+                if broken_line_length and \
+                   parent_index - last_rev_index > broken_line_length:
+                    line_range.append(last_rev_index+1)
+                else:
+                    line_range.extend(range(last_rev_index+1, parent_index))
+            
+            col_index = _find_free_column(columns,
+                                          empty_column,
+                                          col_search_order,
+                                          line_range)
+            node = (col_index, color)
+            for rev_index in branch_line:
+                linegraph[rev_index][1] = node
+                columns[col_index][rev_index] = True
+            
+            for rev_index in branch_line:
+                (sequence_number,
+                     revid,
+                     merge_depth,
+                     revno_sequence,
+                     end_of_merge) = merge_sorted_revisions[rev_index]
+                
+                linegraph[rev_index][4] = graph_children[revid]
+                col_index = linegraph[rev_index][1][0]
+                
+                for parent_revid in graph_parents[revid]:
+                    if parent_revid in revid_index:
+                        
+                        parent_index = revid_index[parent_revid]                            
+                        parent_node = linegraph[parent_index][1]
+                        if parent_node:
+                            parent_col_index = parent_node[0]
+                        else:
+                            parent_col_index = None
                         col_search_order = \
                                 _line_col_search_order(columns,
                                                        parent_col_index,
                                                        col_index)
-                        parent_col_line_index = \
-                            _find_free_column(columns,
-                                              empty_column,
-                                              col_search_order,
-                                              (parent_index - 1,))
-                        _mark_column_as_used(columns,
-                                             parent_col_line_index,
-                                             (parent_index - 1,))
-                        lines.append((rev_index,
-                                      parent_index,
-                                      (child_line_col_index,
-                                       parent_col_line_index)))
-                    else :
-                        line_col_index = col_index
-                        if parent_index - rev_index >1:
-                            line_range = range(rev_index + 1, parent_index)
-                            line_col_index = \
-                                _find_free_column(columns,
-                                                  empty_column,
-                                                  col_search_order,
-                                                  line_range)
-                            _mark_column_as_used(columns,
-                                                 line_col_index,
-                                                 line_range)
-                        lines.append((rev_index,
-                                      parent_index,
-                                      (line_col_index,)))
-    
-    for (child_index, parent_index, line_col_indexes) in lines:
-        (child_col_index, child_color) = linegraph[child_index][1]
-        (parent_col_index, parent_color) = linegraph[parent_index][1]
+                            
+                        # If this line is really long, break it.
+                        if len(branch_id) > 0 and \
+                           broken_line_length and \
+                           parent_index - rev_index > broken_line_length:
+                            child_line_col_index = \
+                                _find_free_column(columns,
+                                                  empty_column,
+                                                  col_search_order,
+                                                  (rev_index + 1,))
+                            _mark_column_as_used(columns,
+                                                 child_line_col_index,
+                                                 (rev_index + 1,))
+                            
+                            # Recall _line_col_search_order to reset it back to
+                            # the beging.
+                            col_search_order = \
+                                    _line_col_search_order(columns,
+                                                           parent_col_index,
+                                                           col_index)
+                            parent_col_line_index = \
+                                _find_free_column(columns,
+                                                  empty_column,
+                                                  col_search_order,
+                                                  (parent_index - 1,))
+                            _mark_column_as_used(columns,
+                                                 parent_col_line_index,
+                                                 (parent_index - 1,))
+                            lines.append((rev_index,
+                                          parent_index,
+                                          (child_line_col_index,
+                                           parent_col_line_index)))
+                        else :
+                            line_col_index = col_index
+                            if parent_index - rev_index >1:
+                                line_range = range(rev_index + 1, parent_index)
+                                line_col_index = \
+                                    _find_free_column(columns,
+                                                      empty_column,
+                                                      col_search_order,
+                                                      line_range)
+                                _mark_column_as_used(columns,
+                                                     line_col_index,
+                                                     line_range)
+                            lines.append((rev_index,
+                                          parent_index,
+                                          (line_col_index,)))
         
-        if len(line_col_indexes) == 1:
-            if parent_index - child_index == 1:
-                linegraph[child_index][2].append(
-                    (child_col_index,
-                     parent_col_index,
-                     parent_color))
+        for (child_index, parent_index, line_col_indexes) in lines:
+            (child_col_index, child_color) = linegraph[child_index][1]
+            (parent_col_index, parent_color) = linegraph[parent_index][1]
+            
+            if len(line_col_indexes) == 1:
+                if parent_index - child_index == 1:
+                    linegraph[child_index][2].append(
+                        (child_col_index,
+                         parent_col_index,
+                         parent_color))
+                else:
+                    # line from the child's column to the lines column
+                    linegraph[child_index][2].append(
+                        (child_col_index,
+                         line_col_indexes[0],
+                         parent_color))
+                    # lines down the line's column
+                    for line_part_index in range(child_index+1, parent_index-1):
+                        linegraph[line_part_index][2].append(
+                            (line_col_indexes[0],   
+                             line_col_indexes[0],
+                             parent_color))
+                    # line from the line's column to the parent's column
+                    linegraph[parent_index-1][2].append(
+                        (line_col_indexes[0],
+                         parent_col_index,
+                         parent_color))
             else:
+                # Broken line
                 # line from the child's column to the lines column
                 linegraph[child_index][2].append(
                     (child_col_index,
                      line_col_indexes[0],
                      parent_color))
-                # lines down the line's column
-                for line_part_index in range(child_index+1, parent_index-1):
-                    linegraph[line_part_index][2].append(
-                        (line_col_indexes[0],   
-                         line_col_indexes[0],
-                         parent_color))
+                # Broken line end
+                linegraph[child_index+1][2].append(
+                    (line_col_indexes[0],
+                     None,
+                     parent_color))
+                
+                # Broken line end 
+                linegraph[parent_index-2][2].append(
+                    (None,
+                     line_col_indexes[1],
+                     parent_color))
                 # line from the line's column to the parent's column
                 linegraph[parent_index-1][2].append(
-                    (line_col_indexes[0],
+                    (line_col_indexes[1],
                      parent_col_index,
                      parent_color))
-        else:
-            # Broken line
-            # line from the child's column to the lines column
-            linegraph[child_index][2].append(
-                (child_col_index,
-                 line_col_indexes[0],
-                 parent_color))
-            # Broken line end
-            linegraph[child_index+1][2].append(
-                (line_col_indexes[0],
-                 None,
-                 parent_color))
-            
-            # Broken line end 
-            linegraph[parent_index-2][2].append(
-                (None,
-                 line_col_indexes[1],
-                 parent_color))
-            # line from the line's column to the parent's column
-            linegraph[parent_index-1][2].append(
-                (line_col_indexes[1],
-                 parent_col_index,
-                 parent_color))
-            
+        return (linegraph, revid_index, len(columns))
+    else:
+        return (linegraph, revid_index, 0)
     
-    return (linegraph, revid_index, len(columns))
 
 def _branch_line_col_search_order(columns, parent_col_index):
     for col_index in range(parent_col_index, len(columns)):

=== modified file 'branchview/treeview.py'
--- branchview/treeview.py	2008-02-14 19:01:19 +0000
+++ branchview/treeview.py	2008-02-14 19:59:06 +0000
@@ -49,11 +49,17 @@
                     gobject.PARAM_READABLE),
 
         'revno-column-visible': (gobject.TYPE_BOOLEAN,
-                                 'Revision number',
+                                 'Revision number column',
                                  'Show revision number column',
                                  True,
                                  gobject.PARAM_READWRITE),
 
+        'graph-column-visible': (gobject.TYPE_BOOLEAN,
+                                 'Graph column',
+                                 'Show graph column',
+                                 True,
+                                 gobject.PARAM_READWRITE),
+
         'date-column-visible': (gobject.TYPE_BOOLEAN,
                                  'Date',
                                  'Show date column',
@@ -64,7 +70,13 @@
                     'Compact view',
                     'Break ancestry lines to save space',
                     True,
-                    gobject.PARAM_CONSTRUCT | gobject.PARAM_READWRITE)
+                    gobject.PARAM_CONSTRUCT | gobject.PARAM_READWRITE),
+
+        'mainline-only': (gobject.TYPE_BOOLEAN,
+                    'Mainline only',
+                    'Only show the mainline history.',
+                    False,
+                    gobject.PARAM_CONSTRUCT | gobject.PARAM_READWRITE),
 
     }
 
@@ -100,7 +112,8 @@
                 lambda x: self.loading_msg_box.hide())
 
         self.scrolled_window = gtk.ScrolledWindow()
-        self.scrolled_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC,
+                                        gtk.POLICY_AUTOMATIC)
         self.scrolled_window.set_shadow_type(gtk.SHADOW_IN)
         self.scrolled_window.show()
         self.pack_start(self.scrolled_window, expand=True, fill=True)
@@ -123,10 +136,14 @@
     def do_get_property(self, property):
         if property.name == 'revno-column-visible':
             return self.revno_column.get_visible()
+        elif property.name == 'graph-column-visible':
+            return self.graph_column.get_visible()
         elif property.name == 'date-column-visible':
             return self.date_column.get_visible()
         elif property.name == 'compact':
             return self.compact
+        elif property.name == 'mainline-only':
+            return self.mainline_only
         elif property.name == 'branch':
             return self.branch
         elif property.name == 'revision':
@@ -143,10 +160,14 @@
     def do_set_property(self, property, value):
         if property.name == 'revno-column-visible':
             self.revno_column.set_visible(value)
+        elif property.name == 'graph-column-visible':
+            self.graph_column.set_visible(value)
         elif property.name == 'date-column-visible':
             self.date_column.set_visible(value)
         elif property.name == 'compact':
             self.compact = value
+        elif property.name == 'mainline-only':
+            self.mainline_only = value
         elif property.name == 'branch':
             self.branch = value
         elif property.name == 'revision':
@@ -259,12 +280,16 @@
             broken_line_length = 32
         else:
             broken_line_length = None
+        
+        show_graph = self.graph_column.get_visible()
 
         self.branch.lock_read()
         (linegraphdata, index, columns_len) = linegraph(self.branch.repository,
                                                         self.start,
                                                         self.maxnum, 
-                                                        broken_line_length)
+                                                        broken_line_length,
+                                                        show_graph,
+                                                        self.mainline_only)
 
         self.model = TreeModel(self.branch, linegraphdata)
         self.graph_cell.columns_len = columns_len

=== modified file 'revbrowser.py'
--- revbrowser.py	2008-02-14 19:01:19 +0000
+++ revbrowser.py	2008-02-14 19:59:06 +0000
@@ -52,6 +52,9 @@
         # Set properties
         self.set_default_size(600, 400)
         self.vbox.set_spacing(3)
+        self.treeview.set_property('graph-column-visible', False)
+        self.treeview.set_property('date-column-visible', True)
+        self.treeview.set_property('mainline-only', True)
         
         # Construct the dialog
         self.action_area.pack_end(self._button_select)

